@@ -172,9 +172,7 @@ def is_line_continuation(line: str) -> bool:
     would result in a set of N/2 slashes not an escape)
 
     :param line: A properties line
-    :type line: str
     :return: Does *line* end with a line continuation
-    :rtype: Boolean
     """
     pos = -1
     count = 0
@@ -188,14 +186,12 @@ def is_line_continuation(line: str) -> bool:
     return (count % 2) == 1  # Odd is a line continuation, even is not
 
 
-def get_comment_one_line(line):
+def get_comment_one_line(line: str) -> str | None:
     """
     Determine whether a *line* is a one-line comment.
 
     :param line: A properties line
-    :type line: unicode
     :return: True if line is a one-line comment
-    :rtype: bool
     """
     stripped = line.strip()
     line_starters = ("#", "!", "//", ";")
@@ -212,9 +208,7 @@ def get_comment_start(line: str) -> str | None:
     Determine whether a *line* starts a new multi-line comment.
 
     :param line: A properties line
-    :type line: unicode
     :return: True if line starts a new multi-line comment
-    :rtype: bool
     """
     stripped = line.strip()
     if stripped.startswith("/*") and not stripped.endswith("*/"):
@@ -227,9 +221,7 @@ def get_comment_end(line: str) -> str | None:
     Determine whether a *line* ends a new multi-line comment.
 
     :param line: A properties line
-    :type line: unicode
     :return: True if line ends a new multi-line comment
-    :rtype: bool
     """
     stripped = line.strip()
     if not stripped.startswith("/*") and stripped.endswith("*/"):
@@ -242,9 +234,7 @@ def _key_strip(key: str) -> str:
     Cleanup whitespace found around a key.
 
     :param key: A properties key
-    :type key: str
     :return: Key without any unneeded whitespace
-    :rtype: str
     """
     newkey = key.rstrip()
     # If string now ends in \ we put back the whitespace that was escaped
@@ -297,16 +287,13 @@ class Dialect:
         """
         Find the type and position of the delimiter in a property line.
 
-        Property files can be delimited by "=", ":" or whitespace (space for now).
+        Property files can be delimited by "=", ":" or whitespace (any whitespace character).
         We find the position of each delimiter, then find the one that appears
         first.
 
         :param line: A properties line
-        :type line: str
         :param delimiters: valid delimiters
-        :type delimiters: list
         :return: delimiter character and offset within *line*
-        :rtype: Tuple (delimiter char, Offset Integer)
         """
         delimiter_dict = {}
         for delimiter in cls.delimiters:
@@ -325,12 +312,21 @@ class Dialect:
                 start_pos += 1
         # Find the position of each delimiter type
         for delimiter, value in delimiters.items():
-            pos = line.find(delimiter, start_pos)
-            while pos != -1:
-                if value == -1 and line[pos - 1] != "\\":
-                    delimiters[delimiter] = pos
-                    break
-                pos = line.find(delimiter, pos + 1)
+            if delimiter == " ":
+                # For whitespace delimiter, find any whitespace character (tab, space, etc.)
+                pos = start_pos
+                while pos < len(line):
+                    if line[pos].isspace() and (pos == 0 or line[pos - 1] != "\\"):
+                        delimiters[delimiter] = pos
+                        break
+                    pos += 1
+            else:
+                pos = line.find(delimiter, start_pos)
+                while pos != -1:
+                    if value == -1 and line[pos - 1] != "\\":
+                        delimiters[delimiter] = pos
+                        break
+                    pos = line.find(delimiter, pos + 1)
         # Find the first delimiter
         mindelimiter = None
         minpos = -1
@@ -430,7 +426,7 @@ class DialectXWiki(DialectJava):
 
     @staticmethod
     def encode(string, encoding=None):
-        return xwiki_properties_encode(string or "", encoding)
+        return xwiki_properties_encode(string or "", encoding)  # ty:ignore[invalid-argument-type]
 
     @staticmethod
     def decode(string):
@@ -483,8 +479,8 @@ class DialectGwt(DialectJavaUtf8):
     @classmethod
     def get_key_cldr_name(cls, key):
         match = cls.plural_regex.fullmatch(key)
-        basekey = match.group(1)
-        variant = match.group(2)
+        basekey = match.group(1)  # ty:ignore[possibly-missing-attribute]
+        variant = match.group(2)  # ty:ignore[possibly-missing-attribute]
         if variant is None:
             variant = ""
 
@@ -499,7 +495,7 @@ class DialectGwt(DialectJavaUtf8):
         return [y for _x, y in cls.gwt_plural_categories]
 
     @classmethod
-    def get_key(cls, key, variant):
+    def get_key(cls, key, variant) -> str:
         variant = cls.cldr2gwt.get(variant)
 
         # Some sanity checks
@@ -680,7 +676,7 @@ class DialectStringsUtf8(DialectStrings):
 class proppluralunit(base.TranslationUnit):
     KEY = "other"
 
-    def __init__(self, source="", personality="java"):
+    def __init__(self, source="", personality="java") -> None:
         """Construct a blank propunit."""
         self.personality = get_dialect(personality)
         super().__init__(source)
@@ -699,7 +695,7 @@ class proppluralunit(base.TranslationUnit):
         return [key for key in data.cldr_plural_categories if key in existing]
 
     def _get_source_mapping(self):
-        cldr_mapping = proppluralunit._get_language_mapping(self._store.sourcelanguage)
+        cldr_mapping = proppluralunit._get_language_mapping(self._store.sourcelanguage)  # ty:ignore[possibly-missing-attribute]
         if cldr_mapping:
             return cldr_mapping
         return self._get_existing_mapping()
@@ -737,7 +733,7 @@ class proppluralunit(base.TranslationUnit):
     def _get_ordered_units(self):
         # Used for str (GWT order)
         mapping = self.personality.get_expand_output_target_mapping(
-            self._store.get_plural_tags()
+            self._store.get_plural_tags()  # ty:ignore[possibly-missing-attribute]
         )
         names = [
             name for name in self.personality.get_cldr_names_order() if name in mapping
@@ -749,7 +745,7 @@ class proppluralunit(base.TranslationUnit):
             return len(self.units) > 1
         return key in self.units
 
-    def settarget(self, text):
+    def settarget(self, text) -> None:
         mapping = None
         if isinstance(text, multistring):
             strings = [str(x) for x in text.strings]
@@ -760,7 +756,7 @@ class proppluralunit(base.TranslationUnit):
         else:
             strings = [text]
         if mapping is None:
-            mapping = self._store.get_plural_tags()
+            mapping = self._store.get_plural_tags()  # ty:ignore[possibly-missing-attribute]
 
         strings = self._get_strings(strings, mapping)
         units = self._get_units(mapping)
@@ -775,7 +771,7 @@ class proppluralunit(base.TranslationUnit):
             b.target = a
 
     def gettarget(self):
-        ll = [x.target for x in self._get_units(self._store.get_plural_tags())]
+        ll = [x.target for x in self._get_units(self._store.get_plural_tags())]  # ty:ignore[possibly-missing-attribute]
         if len(ll) > 1:
             return multistring(ll)
         return ll[0]
@@ -788,7 +784,7 @@ class proppluralunit(base.TranslationUnit):
             return multistring(ll)
         return ll[0]
 
-    def setsource(self, text):
+    def setsource(self, text) -> None:
         mapping = None
         if isinstance(text, multistring):
             strings = text.strings
@@ -817,7 +813,7 @@ class proppluralunit(base.TranslationUnit):
         value = self._get_source_unit().value
         return multistring(value) if value is not None else None
 
-    def setvalue(self, value):
+    def setvalue(self, value) -> None:
         if isinstance(value, multistring):
             strings = value.strings
         elif isinstance(value, list):
@@ -831,7 +827,7 @@ class proppluralunit(base.TranslationUnit):
     def getcomments(self):
         return self._get_source_unit().comments
 
-    def setcomments(self, comments):
+    def setcomments(self, comments) -> None:
         self._get_source_unit().comments = comments
 
     comments = property(getcomments, setcomments)
@@ -839,7 +835,7 @@ class proppluralunit(base.TranslationUnit):
     def getdelimiter(self):
         return self._get_source_unit().delimiter
 
-    def setdelimiter(self, delimiter):
+    def setdelimiter(self, delimiter) -> None:
         self._get_source_unit().delimiter = delimiter
 
     delimiter = property(getdelimiter, setdelimiter)
@@ -850,10 +846,10 @@ class proppluralunit(base.TranslationUnit):
     def getlocations(self):
         return self._get_source_unit().getlocations()
 
-    def add_unit(self, unit, variant):
+    def add_unit(self, unit, variant) -> None:
         self.units[variant] = unit
 
-    def isblank(self):
+    def isblank(self) -> bool:
         """
         Returns whether this is a blank element, containing only
         comments.
@@ -866,7 +862,7 @@ class proppluralunit(base.TranslationUnit):
     def getid(self):
         return self.name
 
-    def setid(self, value):
+    def setid(self, value) -> None:
         self.name = value
 
     @property
@@ -874,10 +870,10 @@ class proppluralunit(base.TranslationUnit):
         return self._get_source_unit().missing
 
     @missing.setter
-    def missing(self, missing):
+    def missing(self, missing) -> None:
         self._get_source_unit().missing = missing
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         Convert to a string. Double check that unicode is handled
         somehow here.
@@ -934,7 +930,7 @@ class propunit(base.TranslationUnit):
     associated.
     """
 
-    def __init__(self, source="", personality="java"):
+    def __init__(self, source="", personality="java") -> None:
         """Construct a blank propunit."""
         self.personality = get_dialect(personality)
         super().__init__(source)
@@ -962,11 +958,11 @@ class propunit(base.TranslationUnit):
         )
 
     @missing.setter
-    def missing(self, missing):
+    def missing(self, missing) -> None:
         self.explicitly_missing = missing
 
     @staticmethod
-    def get_missing_part():
+    def get_missing_part() -> str:
         """Return the string representing a missing translation."""
         return ""
 
@@ -976,7 +972,7 @@ class propunit(base.TranslationUnit):
         return line
 
     @staticmethod
-    def represents_missing(line):
+    def represents_missing(line) -> bool:
         """The line represents a missing translation."""
         return False
 
@@ -985,7 +981,7 @@ class propunit(base.TranslationUnit):
         return self.personality.decode(self.value)
 
     @source.setter
-    def source(self, source):
+    def source(self, source) -> None:
         self._rich_source = None
         self.value = self.personality.encode(source or "", self.encoding)
 
@@ -994,7 +990,7 @@ class propunit(base.TranslationUnit):
         return re.sub(r"\\ ", " ", self.personality.decode(self.translation))
 
     @target.setter
-    def target(self, target):
+    def target(self, target) -> None:
         self._rich_target = None
         self.translation = self.personality.encode(target or "", self.encoding)
         self.explicitly_missing = not bool(target)
@@ -1005,7 +1001,7 @@ class propunit(base.TranslationUnit):
             return self._store.encoding
         return self.personality.default_encoding
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Convert to a string."""
         return self.getoutput()
 
@@ -1039,7 +1035,7 @@ class propunit(base.TranslationUnit):
     def getlocations(self):
         return [self.name]
 
-    def addnote(self, text, origin=None, position="append"):
+    def addnote(self, text, origin=None, position="append") -> None:
         if origin in {"programmer", "developer", "source code", None}:
             if get_comment_one_line(text) is None and get_comment_start(text) is None:
                 text = f"/* {text} */" if "\n" in text else f"// {text}"
@@ -1073,10 +1069,10 @@ class propunit(base.TranslationUnit):
             return "\n".join(output)
         return super().getnotes(origin)
 
-    def removenotes(self, origin=None):
+    def removenotes(self, origin=None) -> None:
         self.comments = []
 
-    def isblank(self):
+    def isblank(self) -> bool:
         """Returns whether this is a blank element, containing only comments."""
         return not (self.name or self.value)
 
@@ -1086,7 +1082,7 @@ class propunit(base.TranslationUnit):
     def getid(self):
         return self.name
 
-    def setid(self, value):
+    def setid(self, value) -> None:
         self.name = value
 
 
@@ -1097,12 +1093,12 @@ class xwikiunit(propunit):
             2. missing translations are output with a dedicated "### Missing: " prefix.
     """
 
-    def __init__(self, source="", personality="xwiki"):
+    def __init__(self, source="", personality="xwiki") -> None:
         super().__init__(source, personality)
         self.output_missing = True
 
     @staticmethod
-    def get_missing_part():
+    def get_missing_part() -> str:
         """Return the string representing a missing translation."""
         return "### Missing: "
 
@@ -1122,7 +1118,7 @@ class propfile(base.TranslationStore):
 
     UnitClass = propunit
 
-    def __init__(self, inputfile=None, personality="java", encoding=None):
+    def __init__(self, inputfile=None, personality="java", encoding=None) -> None:
         """Construct a propfile, optionally reading in from inputfile."""
         super().__init__()
         self.personality = get_dialect(personality)
@@ -1134,7 +1130,7 @@ class propfile(base.TranslationStore):
             self.parse(propsrc)
             self.makeindex()
 
-    def parse(self, propsrc):
+    def parse(self, propsrc) -> None:  # ty:ignore[invalid-method-override]
         """Read the source of a properties file in and include them as units."""
         text, encoding = self.detect_encoding(
             propsrc,
@@ -1151,8 +1147,9 @@ class propfile(base.TranslationStore):
         inmultilinevalue = False
         inmultilinecomment = False
         was_header = False
+        unit_start_line = 1
 
-        for line in propsrc.split("\n"):
+        for linenum, line in enumerate(propsrc.split("\n"), start=1):  # ty:ignore[possibly-missing-attribute]
             # handle multiline value if we're in one
             line = rstripeol(line)
             if inmultilinevalue:
@@ -1167,8 +1164,10 @@ class propfile(base.TranslationStore):
                 if not inmultilinevalue:
                     # we're finished, add it to the list...
                     newunit.value = self.personality.value_strip(newunit.value)
+                    newunit._line_number = unit_start_line
                     self.addunit(newunit)
                     newunit = self.UnitClass("", self.personality.name)
+                    unit_start_line = linenum + 1
             # otherwise, this could be a comment
             # FIXME handle // inline comments
             elif (
@@ -1190,14 +1189,18 @@ class propfile(base.TranslationStore):
                 # this is a blank line...
                 # avoid adding comment only units
                 if newunit.name:
+                    newunit._line_number = unit_start_line
                     self.addunit(newunit)
                     newunit = self.UnitClass("", self.personality.name)
+                    unit_start_line = linenum + 1
                 else:
                     newunit.comments.append("")
 
                 if not was_header and str(newunit).strip():
+                    newunit._line_number = unit_start_line
                     self.addunit(newunit)
                     newunit = self.UnitClass("", self.personality.name)
+                    unit_start_line = linenum + 1
                     was_header = True
 
             else:
@@ -1224,8 +1227,10 @@ class propfile(base.TranslationStore):
                     for comment in inline_comments:
                         if comment not in self.personality.drop_comments:
                             newunit.comments.append(comment)
+                    newunit._line_number = unit_start_line
                     self.addunit(newunit)
                     newunit = self.UnitClass("", self.personality.name)
+                    unit_start_line = linenum + 1
                 else:
                     newunit.name = self.personality.key_strip(line[:delimiter_pos])
                     newunit.missing = ismissing
@@ -1246,16 +1251,19 @@ class propfile(base.TranslationStore):
                         )
                     else:
                         newunit.value = self.personality.value_strip(value_part)
+                        newunit._line_number = unit_start_line
                         self.addunit(newunit)
                         newunit = self.UnitClass("", self.personality.name)
+                        unit_start_line = linenum + 1
         # see if there is a leftover one...
         if inmultilinevalue or any(newunit.comments):
+            newunit._line_number = unit_start_line
             self.addunit(newunit)
 
         if self.personality.has_plurals:
             self.fold()
 
-    def fold(self):
+    def fold(self) -> None:
         old_units = self.units
         self.units = []
         plurals = {}
@@ -1274,7 +1282,7 @@ class propfile(base.TranslationStore):
             # Put the unit
             plurals[key].add_unit(unit, variant)
 
-    def serialize(self, out):
+    def serialize(self, out) -> None:
         """Write the units back to file."""
         # Thanks to iterencode, a possible BOM is written only once
         for chunk in iterencode(
@@ -1288,7 +1296,7 @@ class xwikifile(propfile):
     Extensions = ["properties"]
     UnitClass = xwikiunit
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "xwiki"
         kwargs["encoding"] = "iso-8859-1"
         super().__init__(*args, **kwargs)
@@ -1298,7 +1306,7 @@ class javafile(propfile):
     Name = "Java Properties"
     Extensions = ["properties"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "java"
         kwargs["encoding"] = "auto"
         super().__init__(*args, **kwargs)
@@ -1308,7 +1316,7 @@ class javautf8file(propfile):
     Name = "Java Properties (UTF-8)"
     Extensions = ["properties"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "java-utf8"
         kwargs["encoding"] = "utf-8"
         super().__init__(*args, **kwargs)
@@ -1318,7 +1326,7 @@ class javautf16file(propfile):
     Name = "Java Properties (UTF-16)"
     Extensions = ["properties"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "java-utf16"
         kwargs["encoding"] = "utf-16"
         super().__init__(*args, **kwargs)
@@ -1328,7 +1336,7 @@ class gwtfile(propfile):
     Name = "Gwt Properties"
     Extensions = ["properties"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "gwt"
         kwargs["encoding"] = "utf-8"
         super().__init__(*args, **kwargs)
@@ -1338,7 +1346,7 @@ class stringsfile(propfile):
     Name = "OS X Strings"
     Extensions = ["strings"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "strings"
         super().__init__(*args, **kwargs)
 
@@ -1347,7 +1355,7 @@ class stringsutf8file(stringsfile):
     Name = "OS X Strings (UTF-8)"
     Extensions = ["strings"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "strings-utf8"
         kwargs["encoding"] = "utf-8"
         super().__init__(*args, **kwargs)
@@ -1357,7 +1365,7 @@ class joomlafile(propfile):
     Name = "Joomla Translations"
     Extensions = ["ini"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "joomla"
         super().__init__(*args, **kwargs)
 
@@ -1402,7 +1410,7 @@ class XWikiPageProperties(xwikifile):
     </xwikidoc>
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         kwargs["personality"] = "xwiki"
         kwargs["encoding"] = "utf-8"
         self.root = None
@@ -1415,16 +1423,16 @@ class XWikiPageProperties(xwikifile):
     def get_parser():
         return etree.XMLParser(strip_cdata=False, resolve_entities=False)
 
-    def extract_language(self):
-        language_node = self.root.find("language")
+    def extract_language(self) -> None:
+        language_node = self.root.find("language")  # ty:ignore[possibly-missing-attribute]
         if language_node is not None and language_node.text:
             self.setsourcelanguage(language_node.text)
         else:
-            language_node = self.root.find("defaultLanguage")
+            language_node = self.root.find("defaultLanguage")  # ty:ignore[possibly-missing-attribute]
             if language_node is not None and language_node.text:
                 self.setsourcelanguage(language_node.text)
 
-    def parse(self, propsrc):
+    def parse(self, propsrc) -> None:  # ty:ignore[invalid-method-override]
         if propsrc != b"\n":
             self.root = etree.XML(propsrc, self.get_parser())
             content = "".join(self.root.find("content").itertext())
@@ -1432,7 +1440,7 @@ class XWikiPageProperties(xwikifile):
             self.extract_language()
             super().parse(content)
 
-    def set_xwiki_xml_attributes(self, newroot):
+    def set_xwiki_xml_attributes(self, newroot) -> None:
         for child in newroot.findall("object"):
             newroot.remove(child)
         for child in newroot.findall("attachment"):
@@ -1448,21 +1456,21 @@ class XWikiPageProperties(xwikifile):
         if language_node.text:
             newroot.set("locale", language_node.text)
 
-    def write_xwiki_xml(self, newroot, out):
+    def write_xwiki_xml(self, newroot, out) -> None:
         xml_content = etree.tostring(newroot, encoding=self.encoding, method="xml")
         out.write(self.XML_HEADER.encode(self.encoding))
         out.write(xml_content)
         out.write(b"\n")
 
-    def serialize(self, out):
+    def serialize(self, out) -> None:
         if self.root is None:
             self.root = etree.XML(self.XWIKI_BASIC_XML, self.get_parser())
         newroot = deepcopy(self.root)
         # We add a line break to ensure to have a line break before
         # closing of content tag.
-        newroot.find("content").text = (
-            "".join(unit.getoutput() for unit in self.units).strip() + "\n"
-        )
+        newroot.find(
+            "content"
+        ).text = f"{''.join(unit.getoutput() for unit in self.units).strip()}\n"
         # We only modify the XML attributes if we are editing a translation file
         # if we are editing the source file we should not modify it.
         if not self.is_source_file():
@@ -1482,7 +1490,7 @@ class XWikiFullPage(XWikiPageProperties):
 
     Name = "XWiki Full Page"
 
-    def parse(self, propsrc):
+    def parse(self, propsrc) -> None:  # ty:ignore[invalid-method-override]
         if propsrc != b"\n":
             self.root = etree.XML(propsrc, self.get_parser())
             content = "".join(self.root.find("content").itertext()).replace("\n", "\\n")
@@ -1501,7 +1509,7 @@ class XWikiFullPage(XWikiPageProperties):
         translation = unit.personality.encode(unit.target, unit.encoding)
         return translation or value
 
-    def serialize(self, out):
+    def serialize(self, out) -> None:
         unit_title = self.findid("title")
         unit_content = self.findid("content")
         if self.root is None:
